//
//  SSMotionEffectWithShadow.m
//  TextInMotion
//
//  From Introduction to UIMotionEffect by Ash Furrow of teehan+lax
//  http://www.teehanlax.com/blog/introduction-to-uimotioneffect/
//  A shadow motion could be added in keyPathsAndRelativeValuesForViewerOffset as shown elsewhere
//
//  Created by Bertils mini on 2014-01-11.
//  Copyright (c) 2014 Sork Software. All rights reserved.
//

#import "SSMotionEffectWithShadow.h"
#import "SSViewController.h"

@implementation SSMotionEffectWithShadow

#pragma mark - NSCopying and NSCoding Methods

- (id)copyWithZone:(NSZone *)zone {
    SSMotionEffectWithShadow *otherEffect = [super copyWithZone:zone];
    
    if (otherEffect) {
        otherEffect->_minimumRelativeValue = self.minimumRelativeValue;
        otherEffect->_maximumRelativeValue = self.maximumRelativeValue;
        otherEffect->_type = self.type;
        otherEffect->_keyPath = self.keyPath;
    }
    
    return otherEffect;
}

- (void)encodeWithCoder:(NSCoder *)aCoder {
    [super encodeWithCoder:aCoder];
    
    [aCoder encodeObject:@(self.minimumRelativeValue) forKey:@"minimumRelativeValue"];
    [aCoder encodeObject:@(self.maximumRelativeValue) forKey:@"maximumRelativeValue"];
    [aCoder encodeObject:@(self.type) forKey:@"type"];
    [aCoder encodeObject:self.keyPath forKey:@"keyPath"];
}

- (id)initWithCoder:(NSCoder *)aDecoder {
    if (!(self = [super initWithCoder:aDecoder])) return nil;
    
    _minimumRelativeValue = [[aDecoder decodeObjectForKey:@"minimumRelativeValue"] floatValue];
    _maximumRelativeValue = [[aDecoder decodeObjectForKey:@"maximumRelativeValue"] floatValue];
    _type = (UIInterpolatingMotionEffectType)[[aDecoder decodeObjectForKey:@"type"] integerValue];
    _keyPath = [aDecoder decodeObjectForKey:@"keyPath"];
    
    return self;
}

#pragma mark - Public Methods

- (instancetype)initWithKeyPath:(NSString *)keyPath type:(UIInterpolatingMotionEffectType)type {
    if (!(self = [super init])) return nil;
    
    _type = type;
    _keyPath = keyPath;
    
    return self;
}

#pragma mark - UIMotionEffects Methods

// SS modification with position motion
- (NSDictionary *)keyPathsAndRelativeValuesForViewerOffset:(UIOffset)viewerOffset {
    CGFloat offset = self.type == UIInterpolatingMotionEffectTypeTiltAlongHorizontalAxis ? viewerOffset.horizontal : viewerOffset.vertical;
    
    // Notify our VC about a move action and its offsets packed into a dictionary as a point value
    NSValue *offsets = [NSValue valueWithCGPoint:CGPointMake(viewerOffset.horizontal, viewerOffset.vertical)];
    NSDictionary *didMove = @{@"Move" : self, @"Offset" : offsets};
    [[NSNotificationCenter defaultCenter] postNotificationName:MoveNotification object:self userInfo:didMove];
    
    return @{self.keyPath : @(self.maximumRelativeValue * offset)};
}

@end
